@@ -6,26 +6,11 @@ package Tangram::Storage;
 use DBI;
 use Carp;
 use Tangram::Core;
+use Scalar::Util qw(weaken refaddr);
 
 use vars qw( %storage_class );
 
 BEGIN {
-    eval 'use Scalar::Util qw()';
-    if ($@) {
-	*Tangram::refaddr = sub { (shift) + 0 };
-	eval 'use WeakRef';
-	if ($@) {
-	    *Tangram::weaken = sub { };
-	    $Tangram::no_weakrefs = 1;
-	} else {
-	    *Tangram::weaken = \&WeakRef::weaken;
-	    $Tangram::no_weakrefs = 0;
-	}
-    } else {
-	*Tangram::weaken = \&Scalar::Util::weaken;
-	*Tangram::refaddr = \&Scalar::Util::refaddr;
-	$Tangram::no_weakrefs = 0;
-    }
     *pretty = *Tangram::Core::pretty;
 }
 
@@ -177,28 +162,32 @@ sub _open
 		}
 	  }
 	  if ($id) {
-	    $self->{ids}{Tangram::refaddr($obj)} = $id;
+	    $self->{ids}{refaddr($obj)} = $id;
 	  } else {
-	    delete $self->{ids}{Tangram::refaddr($obj)};
+	    delete $self->{ids}{refaddr($obj)};
 	  }
 	};
 
     $self->{get_id} = $schema->{get_id} || sub {
 	  my $obj = shift or warn "no object passed to get_id";
 	  ref $obj or return undef;
-	  my $address = Tangram::refaddr($obj)
+	  my $address = refaddr($obj)
 	      or do { warn "Object $obj has no refaddr(?)";
 		      return undef };
 	  my $id = $self->{ids}{$address};
+	  # refaddr's can be re-used, but weakrefs are magic :-)
+	  if ( $id and !defined $self->{objects}{$id} ) {
+	      delete $self->{ids}{$address};
+	      delete $self->{objects}{$id};
+	      $id = undef;
+	  } elsif ( $id and refaddr($self->{objects}{$id}) != $address ) {
+	      delete $self->{ids}{$address};
+	      $id = undef;
+	  }
 	  if ($Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 2)) {
 		print $Tangram::TRACE "Tangram: $obj is ".($id?"oid $id" : "not in storage")."\n";
 	  }
-	  return undef unless $id;
-	  # refaddr's can be re-used, but weakrefs are magic :-)
-	  return $id if defined($self->{objects}{$id});
-	  delete $self->{ids}{$address};
-	  delete $self->{objects}{$id};
-	  return undef;
+	  return $id;
 	};
 
     return $self;
@@ -974,7 +963,7 @@ sub welcome
     my ($self, $obj, $id) = @_;
     $self->{set_id}->($obj, $id);
 
-    Tangram::weaken( $self->{objects}{$id} = $obj );
+    weaken( $self->{objects}{$id} = $obj );
   }
 
 sub goodbye
